/*
    myrrdin, a 2d tile engine
    Copyright (c) 2008 Friedrich Weber

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the Software), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
 */

module myrrdin.renderer;

import dsfml.window.all;
import dsfml.system.all;
import dsfml.graphics.all;

import myrrdin.consumers.consumer;
import myrrdin.imagecache;
import myrrdin.animatedsprite;

class Renderer {
    public RenderWindow app;
    private Consumer[] draw_consumers;
    private Consumer[] event_consumers;
    
    this(char[] title, int width, int height, int depth=32) {
	this.app = new RenderWindow(VideoMode(width, height, depth), title);
	this.app.setFramerateLimit(40);
    }

    /* add a consumer for event handling and drawing */
    void add_consumer(Consumer consumer) {
	this.add_event_consumer(consumer);
	this.add_draw_consumer(consumer);
    }

    /* add a consumer only for event handling */
    void add_event_consumer(Consumer consumer) {
	this.event_consumers ~= consumer;
    }

    /* add a consumer only for drawing */
    void add_draw_consumer(Consumer consumer) {
	this.draw_consumers ~= consumer;
    }

    /* use this instead of this.app.draw - it contains a hook for animated sprites */
    void draw(AnimatedSprite obj) {
	obj.update();
	this.app.draw(obj);
    }

    void draw(IDrawable obj) {
	this.app.draw(obj);
    }

    void draw(Sprite obj) {
	if(cast(AnimatedSprite)obj !is null) {
	    this.draw(cast(AnimatedSprite)obj);
	}
	else {
	    this.app.draw(obj);
	}
    }

    /* start the mainloop */
    void mainloop() {
	Event evt;

	while(this.app.isOpened()) {
	    // handle all events
	    while(this.app.getEvent(evt)) {
		if (evt.Type == Event.EventType.CLOSED) {
		    this.app.close();
		} else {
		    foreach(Consumer consumer; this.event_consumers) {
			if(consumer.handle_event(evt)) break;
		    }
		}		    
	    }
	    // loop iteration
	    foreach(Consumer consumer; this.draw_consumers) {
		consumer.loop_iteration();
	    }
	    // draw all
	    foreach(Consumer consumer; this.draw_consumers) {
		consumer.draw();
	    }
	    // display all
	    this.app.display();
	}
    }
}
