/*
    myrrdin, a 2d tile engine
    Copyright (c) 2008 Friedrich Weber

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the Software), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
 */

module myrrdin.xmlmap;

import myrrdin.tileset;
import myrrdin.tilemap;

import tango.text.xml.Document;
import tango.text.convert.Integer;
import tango.io.Stdout; // TODO
import myrrdin.tools;
import myrrdin.imagecache;
import dsfml.system.all;
import Text = tango.text.Util;
static import tango.core.Exception;

alias XmlPath!(char).NodeSet NodeSet;
alias Document!(char).Node NodeImpl;
private char[] get_attribute(NodeSet node, char[] name) {
    return node.attribute(name).nodes[0].value;
}

Tileset parse_tileset(Cache cache, char[] content) {
    auto tileset = new Tileset(cache);

    auto doc = new Document!(char);
    doc.parse(content);
    auto root = doc.query["tileset"];   
    auto nodes = root.child.nodes;
    foreach(NodeImpl tiles_node; nodes) {
	tileset.add_tile(Integer.parse(tiles_node.getAttribute("id").value), tiles_node.getAttribute("filename").value);
    }
    return tileset;
}

Tilemap parse_map(Cache cache, char[] content) {
    auto doc = new Document!(char);
    doc.parse(content);
    auto root = doc.query["map"];
    int width = Integer.parse(get_attribute(root, "width"));
    int height = Integer.parse(get_attribute(root, "height"));
    Stdout.formatln("Width: {}, Height: {}", width, height);
    auto tiles_node = root.child.nodes[0];
    char[] tileset_file = tiles_node.getAttribute("tileset").value;
    auto tilemap = new Tilemap(parse_tileset(cache, read_file_contents(tileset_file)), width, height, 32, 32); // TODO: variable tile size
    int layer_id=0;
    foreach(NodeImpl layer_node; tiles_node.query["layer"].nodes) {
	int tile_width = Integer.parse(layer_node.getAttribute("tilewidth").value);
	int tile_height = Integer.parse(layer_node.getAttribute("tileheight").value);
	tilemap.layer_tsizes[layer_id] = Vector2i(tile_width, tile_height);
	char[][] content_ = Text.delimit(Text.trim(layer_node.value), ",");
	int i=0;
	bool running = true;
	for(int x=0; x < width; x++) {
	    for(int y=0; y < height; y++) {
		try {
		    tilemap.map[layer_id][x][y] = Integer.parse(content_[i]);
		}
		catch (tango.core.Exception.ArrayBoundsException) {
		    // array out of bounds ... break
		    running = false;
		    break;
		}
		i++;
	    }
	    if(!running) {
		break;
	    }
	}
	layer_id++;
    }
    return tilemap;
}
